# SPDX-FileCopyrightText: 2019 Brent Rubell for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""
`adafruit_cursorcontrol.cursorcontrol_cursormanager`
================================================================================
Mouse cursor for interaction with CircuitPython UI elements

* Author(s): Brent Rubell

"""
__version__ = "2.7.4"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_CursorControl.git"
